#version 430 core

in vec4 clipSpace;
in vec2 textureCoords;
in vec3 fromLightVector;
in vec4 vertexColor;
in vec3 toCameraVector;
in vec3 vertPos;
in vec3 camPos;

layout(binding=0) uniform sampler2D reflectionTexture;
layout(binding=1) uniform sampler2D refractionTexture;
layout(binding=2) uniform sampler2D dudvMap;
layout(binding=3) uniform sampler2D normalMap;

uniform float moveFactor;

const float waveStrength =0.02;
const float shineDamper = 20.0;
const float reflectivity = 0.2;
const vec3 lightColor = vec3(1.0f, 1.0f, 1.0f);

out vec4 out_Color;

void main()
{
	if(length(camPos-vertPos)>7 || length(camPos-vertPos)<-7 ){discard;}
	// Koordinaten transformieren
	vec2 ndc=(clipSpace.xy/clipSpace.w)/2.0+0.5;
	vec2 refractCoords=vec2(ndc.x, ndc.y);
	vec2 reflectCoords=vec2(ndc.x, -ndc.y);

	// Wellen
	vec2 distortedTexCoords = texture(dudvMap, vec2(textureCoords.x + moveFactor, textureCoords.y)).rg*0.1;
	distortedTexCoords = textureCoords + vec2(distortedTexCoords.x, distortedTexCoords.y+moveFactor);
	vec2 totalDistortion = (texture(dudvMap, distortedTexCoords).rg * 2.0 - 1.0) * waveStrength;

	// Korrekturen
	refractCoords += totalDistortion;
	refractCoords = clamp(refractCoords,0.001,0.999);
	reflectCoords += totalDistortion;
	reflectCoords.x = clamp(reflectCoords.x,0.001,0.999);
	reflectCoords.y = clamp(reflectCoords.y,-0.999,-0.001);

	// Fresnel Effekt
	vec4 reflectColor = texture(reflectionTexture,reflectCoords);
	vec4 refractColor = texture(refractionTexture,refractCoords);
	vec3 viewVector = normalize(toCameraVector);
	float refractiveFactor =dot(viewVector,vec3(0,1,0));
	refractiveFactor = pow(refractiveFactor,0.5);

	// Normal Mapping
	vec4 normalMapColor = texture(normalMap,distortedTexCoords);
	vec3 normal = vec3(normalMapColor.r*2.0-1.0,normalMapColor.b,normalMapColor.g*2.0-1.0);
	normal = normalize(normal);
	out_Color = mix(reflectColor,refractColor,refractiveFactor);

	// specular
	vec3 reflectedLight = reflect(normalize(fromLightVector), normal);
	float specular = max(dot(reflectedLight, viewVector), 0.0);
	specular = pow(specular, shineDamper);
	vec3 specularHighlights = lightColor * specular * reflectivity;
	
	// Blauer Farbstich
	out_Color = mix(out_Color,vec4(0.0,0.3,0.4,1.0),0.2); 

	// Nebel Interpolation
	out_Color = vec4(mix(out_Color.rgb,vec3(0.92f, 0.90f, 1.0f).rgb,vertexColor.r), 0.5);
	
	// specular Highlights
	out_Color += vec4(specularHighlights,0.0f);
	
}